/*******************************************************************************
* File Name: My_CapSense_1_TunerHelper.c
* Version 2.10
*
* Description:
*  This file provides the source code of the Tuner helper APIs for the CapSense CSD 
*  component.
*
* Note:
*
********************************************************************************
* Copyright 2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "My_CapSense_1_TunerHelper.h"

#if (My_CapSense_1_TUNER_API_GENERATE)
    #include "SCB.h"
    #include "SCB_EZI2C.h"
    volatile My_CapSense_1_MAILBOXES My_CapSense_1_mailboxesComm;
#endif  /* (My_CapSense_1_TUNER_API_GENERATE) */

/* `#START My_CapSense_1_TunerHelper_HEADER` */

/* `#END` */

/*******************************************************************************
* Function Name: My_CapSense_1_TunerStart
********************************************************************************
*
* Summary:
*  Initializes the CapSense CSD component and EzI2C communication component to use
*  a mailbox data structure for communication with the Tuner GUI.
*  Start the scanning, after initialization is complete.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  None
*
*******************************************************************************/
void My_CapSense_1_TunerStart(void)
{
    #if (0u != My_CapSense_1_TUNER_API_GENERATE)
		
        /* Init mbx and quick check */
        My_CapSense_1_InitMailbox(&My_CapSense_1_mailboxesComm.csdMailbox);
        My_CapSense_1_mailboxesComm.numMailBoxes = My_CapSense_1_DEFAULT_MAILBOXES_NUMBER;
        
        /* Start CapSense and baselines */
        My_CapSense_1_Start();
        
        /* Initialize baselines */
		#if(0u != My_CapSense_1_CSHL_API_GENERATE)
	        My_CapSense_1_InitializeEnabledBaselines();
		#endif /* (0u != My_CapSense_1_CSHL_API_GENERATE) */
		
        /* Start EzI2C, clears buf pointers */
        SCB_Start();
		
        /* Setup EzI2C buffers */
        SCB_EzI2CSetBuffer1(sizeof(My_CapSense_1_mailboxesComm), sizeof(My_CapSense_1_mailboxesComm),
                                      			   (volatile uint8 *) &My_CapSense_1_mailboxesComm);
        
        /* Starts scan all enabled sensors */
        My_CapSense_1_ScanEnabledWidgets();
    
    #endif  /* (0u != My_CapSense_1_TUNER_API_GENERATE) */
}


/*******************************************************************************
* Function Name: My_CapSense_1_TunerComm
********************************************************************************
*
* Summary:
*  This function is blocking. It waits till scanning loop is completed and applies
*  the new parameters from the Tuner GUI if available (the manual tuning mode only). Updates
*  the enabled baselines and state of widgets. Waits while the Tuner GUI reports that 
*  the mailbox content  could be modified. Then loads the report data into the outbox 
*  and sets a busy flag. Starts a new scanning loop.
*  
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  None
*
*******************************************************************************/
void My_CapSense_1_TunerComm(void)
{
    #if (0u != My_CapSense_1_TUNER_API_GENERATE)
        if (0u == My_CapSense_1_IsBusy())
        {   
            /* Apply new settings */
            #if((My_CapSense_1_TUNING_METHOD != My_CapSense_1__TUNING_NONE) &&\
				(0u != My_CapSense_1_TOTAL_TRACKPAD_GESTURES_COUNT) ||\
		        (My_CapSense_1_TUNING_METHOD == My_CapSense_1__TUNING_MANUAL)) 
     
				My_CapSense_1_ReadMessage(&My_CapSense_1_mailboxesComm.csdMailbox);
				
            #endif  /* ((My_CapSense_1_TUNING_METHOD != My_CapSense_1__TUNING_NONE) &&\
						(0u != My_CapSense_1_TOTAL_TRACKPAD_GESTURES_COUNT) ||\
		                (My_CapSense_1_TUNING_METHOD == My_CapSense_1__TUNING_MANUAL)) */
			
			#if(0u != My_CapSense_1_CSHL_API_GENERATE)
				/* Update all baselines and process all widgets */
            	My_CapSense_1_UpdateEnabledBaselines();
            	My_CapSense_1_ProcessAllWidgets(&My_CapSense_1_mailboxesComm.csdMailbox.outbox);
			#endif /* (0u != My_CapSense_1_CSHL_API_GENERATE) */
			
            My_CapSense_1_PostMessage(&My_CapSense_1_mailboxesComm.csdMailbox);
			
            /* Enable EZI2C interrupts, after scan complete */
            SCB_EnableInt();

            while((My_CapSense_1_mailboxesComm.csdMailbox.type != My_CapSense_1_TYPE_ID) ||
			      (0u != (SCB_EzI2CGetActivity() & SCB_EZI2C_STATUS_BUSY))){}
            
            /* Disable EZI2C interrupts, while scanning */
            SCB_DisableInt();
            
            /* Start scan all sensors */
            My_CapSense_1_ScanEnabledWidgets();
        }
    #endif /* (0u != My_CapSense_1_TUNER_API_GENERATE) */
}


#if ((0u != My_CapSense_1_TUNER_API_GENERATE) && (0u != My_CapSense_1_CSHL_API_GENERATE))
    /*******************************************************************************
    * Function Name: My_CapSense_1_ProcessAllWidgets
    ********************************************************************************
    *
    * Summary:
    *  Calls the required functions to update all the widgets' state:
    *   - My_CapSense_1_GetCentroidPos() - calls only if linear sliders are
    *     available.
    *   - My_CapSense_1_GetRadialCentroidPos() - calls only if the  radial slider is
    *     available.
    *   - My_CapSense_1_GetTouchCentroidPos() - calls only if the  touchpad slider 
    *     available.
    *   - My_CapSense_1_CheckIsAnyWidgetActive();
    *  The results of operations are copied to OUTBOX.
    *   
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    * Global Variables:
    *  My_CapSense_1_OUTBOX outbox - the structure which is used as an output 
    *  buffer to report data to the Tuner GUI.
    *  Update fields:
    *    - position[];
    *    - OnMask[];
	*
	* Global Variables:
	*  None
	*
    *******************************************************************************/
    void My_CapSense_1_ProcessAllWidgets(volatile My_CapSense_1_OUTBOX *outbox)
    {
        uint8  i = 0u;
		
		#if (0u != My_CapSense_1_TOTAL_MATRIX_BUTTONS_COUNT)
        	uint8 mbPositionBuf[My_CapSense_1_MB_POSITION_BUFFER_SIZE];
		#endif /* (0u != My_CapSense_1_TOTAL_MATRIX_BUTTONS_COUNT) */

        #if (0u != My_CapSense_1_TOTAL_TOUCH_PADS_BASIC_COUNT)
            uint16 centroidPosBuf[My_CapSense_1_CENTROID_POSITION_BUFFER_SIZE];
        #endif  /* (0u != My_CapSense_1_TOTAL_TOUCH_PADS_BASIC_COUNT) */
		
		#if(0u != My_CapSense_1_TOTAL_TRACKPAD_GESTURES_COUNT)
            My_CapSense_1_POSITION_STRUCT trackpadPosBuf[My_CapSense_1_CENTROID_POSITION_BUFFER_SIZE];
        #endif  /* (0u != My_CapSense_1_TOTAL_TRACKPAD_GESTURES_COUNT) */
        
        #if ( (0u != My_CapSense_1_TOTAL_RADIAL_SLIDERS_COUNT) || (0u != My_CapSense_1_TOTAL_TOUCH_PADS_COUNT) || \
              (0u != My_CapSense_1_TOTAL_MATRIX_BUTTONS_COUNT))
            uint8 widgetCnt;
        #endif  /* ((0u != My_CapSense_1_TOTAL_RADIAL_SLIDERS_COUNT) || (0u != My_CapSense_1_TOTAL_TOUCH_PADS_COUNT)) || 
                *   (0u != My_CapSense_1_TOTAL_MATRIX_BUTTONS_COUNT)
                */
        
        /* Calculate widget with centroids */
        #if (0u != My_CapSense_1_TOTAL_LINEAR_SLIDERS_COUNT)
            for(; i < My_CapSense_1_TOTAL_LINEAR_SLIDERS_COUNT; i++)
            {
                outbox->centroidPosition[i] = (uint16)My_CapSense_1_GetCentroidPos((uint32)i);
            }
        #endif /* (0u != My_CapSense_1_TOTAL_LINEAR_SLIDERS_COUNT) */
        
        #if (0u != My_CapSense_1_TOTAL_RADIAL_SLIDERS_COUNT)
            widgetCnt = i;
            for(; i < (widgetCnt + My_CapSense_1_TOTAL_RADIAL_SLIDERS_COUNT); i++)
            {
                outbox->centroidPosition[i] = (uint16)My_CapSense_1_GetRadialCentroidPos((uint32)i);
            }
        #endif /* (0u != My_CapSense_1_TOTAL_RADIAL_SLIDERS_COUNT) */
        
        #if (0u != My_CapSense_1_TOTAL_TOUCH_PADS_BASIC_COUNT)
            widgetCnt = i;
            for(; i < (widgetCnt + (My_CapSense_1_TOTAL_TOUCH_PADS_BASIC_COUNT * 2u)); i=i+2u)
            {
                if(My_CapSense_1_GetTouchCentroidPos((uint32)i, centroidPosBuf) == 0u)
                {
                    outbox->centroidPosition[i + My_CapSense_1_FIRST_FINGER_X_INDEX] = My_CapSense_1_MAX_UINT_16;
                    outbox->centroidPosition[i + My_CapSense_1_FIRST_FINGER_Y_INDEX] = My_CapSense_1_MAX_UINT_16;
                }
                else
                {
                    outbox->centroidPosition[i + My_CapSense_1_FIRST_FINGER_X_INDEX] =
					(uint16) centroidPosBuf[My_CapSense_1_FIRST_FINGER_X_INDEX];
                    
					outbox->centroidPosition[i + My_CapSense_1_FIRST_FINGER_Y_INDEX] =
					(uint16) centroidPosBuf[My_CapSense_1_FIRST_FINGER_Y_INDEX];
                }
            }
        #endif /* (0u != My_CapSense_1_TOTAL_TOUCH_PADS_BASIC_COUNT) */
		
		#if (0u != My_CapSense_1_TOTAL_TRACKPAD_GESTURES_COUNT)
			widgetCnt = i;
			for(; i < (widgetCnt + (My_CapSense_1_TOTAL_TRACKPAD_GESTURES_COUNT * 4u)); i=i+4u)
			{
				outbox->fingersNum[My_CapSense_1_DEFAULT_TRACKPAD_INDEX] =
				(uint8)My_CapSense_1_GetDoubleTouchCentroidPos(trackpadPosBuf);
				
				outbox->gestureID[My_CapSense_1_DEFAULT_TRACKPAD_INDEX] =
				(uint8)My_CapSense_1_DecodeAllGestures((uint32)outbox->fingersNum[My_CapSense_1_DEFAULT_TRACKPAD_INDEX], trackpadPosBuf); 

				#if((0u != My_CapSense_1_ST_SCROLL_GEST_ENABLED) || (0u != My_CapSense_1_DT_SCROLL_GEST_ENABLED))
					/*
					Report Scroll count only for scroll and Inertial Scroll gestures.
					For all other gestures report scroll count as zero to tuner.
					 */
					if (((outbox->gestureID[My_CapSense_1_DEFAULT_TRACKPAD_INDEX]&0xF0u) == 0xC0) ||
							((outbox->gestureID[My_CapSense_1_DEFAULT_TRACKPAD_INDEX]&0xF0u) == 0xB0u))
					{
						outbox->scrollCnt = My_CapSense_1_GetScrollCnt();
					}
					else
					{
						outbox->scrollCnt = 0u;
					}
				#endif /* ((0u != My_CapSense_1_ST_SCROLL_GEST_ENABLED) || (0u != My_CapSense_1_DT_SCROLL_GEST_ENABLED)) */

                outbox->centroidPosition[i + My_CapSense_1_FIRST_FINGER_X_INDEX] = My_CapSense_1_MAX_UINT_16;
                outbox->centroidPosition[i + My_CapSense_1_FIRST_FINGER_Y_INDEX] = My_CapSense_1_MAX_UINT_16;
				outbox->centroidPosition[i + My_CapSense_1_SECOND_FINGER_X_INDEX] = My_CapSense_1_MAX_UINT_16;
                outbox->centroidPosition[i + My_CapSense_1_SECOND_FINGER_Y_INDEX] = My_CapSense_1_MAX_UINT_16;

	            if(0u != outbox->fingersNum[My_CapSense_1_DEFAULT_TRACKPAD_INDEX])
	            {
	                outbox->centroidPosition[i + My_CapSense_1_FIRST_FINGER_X_INDEX] =
					(uint16) trackpadPosBuf[My_CapSense_1_FIRST_FINGER_INDEX].x;
	                
					outbox->centroidPosition[i + My_CapSense_1_FIRST_FINGER_Y_INDEX] =
					(uint16) trackpadPosBuf[My_CapSense_1_FIRST_FINGER_INDEX].y;
					
					if(outbox->fingersNum[My_CapSense_1_DEFAULT_TRACKPAD_INDEX] > 1u)
					{
						outbox->centroidPosition[i + My_CapSense_1_SECOND_FINGER_X_INDEX] =
						(uint16) trackpadPosBuf[My_CapSense_1_SECOND_FINGER_INDEX].x;
	                	
						outbox->centroidPosition[i + My_CapSense_1_SECOND_FINGER_Y_INDEX] =
						(uint16) trackpadPosBuf[My_CapSense_1_SECOND_FINGER_INDEX].y;
					}
	            }
			}
		#endif /* (0u != My_CapSense_1_TOTAL_TRACKPAD_GESTURES_COUNT) */

        #if (0u != My_CapSense_1_TOTAL_MATRIX_BUTTONS_COUNT)
            i += My_CapSense_1_TOTAL_BUTTONS_COUNT;
            widgetCnt = 0u;
            for(; widgetCnt < (My_CapSense_1_TOTAL_MATRIX_BUTTONS_COUNT * 2u); widgetCnt += 2u)
            {
                if(My_CapSense_1_GetMatrixButtonPos((uint32)i, mbPositionBuf) == 0u)
                {
                    outbox->mbPosition[widgetCnt] = My_CapSense_1_MAX_UINT_8;
                    outbox->mbPosition[widgetCnt+1u] = My_CapSense_1_MAX_UINT_8;
                }
				else
				{
					outbox->mbPosition[widgetCnt] = mbPositionBuf[0u];
                    outbox->mbPosition[widgetCnt+1u] = mbPositionBuf[1u];
				}
                i += 2u;
            }
        #endif /* (0u != My_CapSense_1_TOTAL_MATRIX_BUTTONS_COUNT) */

		#if(0u != My_CapSense_1_CSHL_API_GENERATE)
	        /* Update On/Off State */
	        (void)My_CapSense_1_CheckIsAnyWidgetActive();

	        /* Copy OnMask */
	        for(i=0u; i < My_CapSense_1_TOTAL_SENSOR_MASK_COUNT; i++)
	        {
	            outbox->onMask[i]  = My_CapSense_1_sensorOnMask[i];
	        }
		#endif /* (0u != My_CapSense_1_CSHL_API_GENERATE) */
		
		/* `#START My_CapSense_1_ProcessAllWidgets_Debug` */

		/* `#END` */
    }


#endif /* ((0u != My_CapSense_1_TUNER_API_GENERATE) && (0u != My_CapSense_1_CSHL_API_GENERATE)) */


/* [] END OF FILE */
